//
// File:        PythonServerLaunch.java
// Package:     gov.llnl.babel.backend.python
// Revision:    @(#) $Revision: 5000 $
// Date:        $Date: 2005-11-01 11:44:31 -0800 (Tue, 01 Nov 2005) $
// Description: Initialize Python before loading the real implementation
// 
// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// UCRL-CODE-2002-054
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

package gov.llnl.babel.backend.python;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.backend.CodeGenerationException;
import gov.llnl.babel.backend.IOR;
import gov.llnl.babel.backend.Utilities;
import gov.llnl.babel.backend.c.C;
import gov.llnl.babel.backend.writers.LanguageWriterForC;
import gov.llnl.babel.symbols.Extendable;
import gov.llnl.babel.symbols.Class;
import gov.llnl.babel.symbols.SymbolID;
import gov.llnl.babel.symbols.Symbol;
import gov.llnl.babel.symbols.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.Collection;
import java.util.HashSet;

public class PythonServerLaunch {
  LanguageWriterForC d_lw = null;
  Class d_cls;
  boolean d_hasStatic;

  PythonServerLaunch(Class cls)
  {
    d_cls = cls;
    d_hasStatic = d_cls.hasStaticMethod(false);
  }

  private void writeIncludes() throws CodeGenerationException
  {
    d_lw.generateInclude("sidl_Python.h", true);
    d_lw.generateInclude("sidl_Loader.h", true);
    d_lw.printlnUnformatted("#include <stdio.h>");
    /*    Iterator i = Utilities.convertIdsToSymbols(argSymbols()).iterator();
          while (i.hasNext()) {
          Symbol sym = (Symbol)i.next();
          if (Symbol.ENUM != sym.getSymbolType()) {
          d_lw.generateInclude(Python.getCHeaderPath(sym, "Module"), true);
          }
          }
          d_lw.generateInclude(Python.getCHeaderPath(d_cls, "Module"), true);
          java.util.Collection parents = d_cls.getParents(true);
          for(Iterator j = parents.iterator(); j.hasNext(); ){
          Extendable parent = (Extendable) j.next();
          d_lw.generateInclude(Python.getCHeaderPath(parent, "Module"), true);
          } 
    */
    d_lw.println();
  }

  private void writeForwardDecls()
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.println(IOR.getEPVName(id) + ";");
    if (d_hasStatic) {
      d_lw.println(IOR.getSEPVName(id) + ";");
    }
    d_lw.println();
  }

  private void writeStaticVars() throws CodeGenerationException
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.println("static void (*s_getEPV)");
    d_lw.tab();
    d_lw.println("(" + IOR.getEPVName(id) + " *);");
    d_lw.backTab();
    if (d_hasStatic) {
      d_lw.println("static void (*s_getSEPV)");
      d_lw.tab();
      d_lw.println("(" + IOR.getSEPVName(id) + " *);");
      d_lw.backTab();
    }
    d_lw.println();

    Python.generateRMIExternStruct(d_cls, d_lw);
    d_lw.println();
    d_lw.println("static struct "+IOR.getSymbolName(id)+"__rmiExternals *s_rmiExternals;");
    d_lw.println();

  }
  
  private void writeInit()
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.println("static void");
    d_lw.println("initImplementation(void) {");
    d_lw.tab();
    d_lw.println(IOR.getExceptionFundamentalType() + "throwaway_exception;");
    d_lw.println("static int notInitialized = 1;");
    d_lw.println("if (notInitialized) {");
    d_lw.tab();
    d_lw.println("sidl_DLL dll = sidl_Loader_findLibrary(\"" + 
                 id.getFullName() + "\",");
    d_lw.tab();
    d_lw.println("\"python/impl\", sidl_Scope_SCLSCOPE,");
    d_lw.println("sidl_Resolve_SCLRESOLVE, &throwaway_exception);");
    d_lw.backTab();
    d_lw.println("if (dll) {");
    d_lw.tab();
    d_lw.println("struct "+IOR.getSymbolName(id)+"__rmiExternals *((*rmi_func)(void));");

    d_lw.println("notInitialized = 0;");
    d_lw.println("sidl_Python_Init();");
    d_lw.println("s_getEPV = (void (*)(" + IOR.getEPVName(id) + " *))");
    d_lw.tab();
    d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
                 Python.getSetEPVName(d_cls.getSymbolID()) + "\", &throwaway_exception);");
    d_lw.backTab();

    d_lw.println("rmi_func = (struct "+IOR.getSymbolName(id)+"__rmiExternals*((*)(void))) ");
    d_lw.tab();
    d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
                 Python.getRMIExternName(d_cls.getSymbolID()) + "\", &throwaway_exception);");
                 
    d_lw.println("if (rmi_func) {");
    d_lw.tab();
    d_lw.println("s_rmiExternals = (*rmi_func)();");
    d_lw.backTab();
    d_lw.println("}"); 
    d_lw.backTab();
    if (d_hasStatic) {
      d_lw.tab();
      d_lw.println("s_getSEPV = (void (*)(" + IOR.getSEPVName(id) + " *))");
      d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
                   Python.getSetSEPVName(d_cls.getSymbolID()) + "\", &throwaway_exception);");
      d_lw.backTab();
    }
    

    d_lw.println("sidl_DLL_deleteRef(dll, &throwaway_exception);");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.println();
  }

  private void writeGetEPV()
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.openCxxExtern();
    d_lw.println("void");
    d_lw.println(IOR.getSetEPVName(id) + "(" +
                 IOR.getEPVName(id) + " *epv) {");
    d_lw.tab();
    d_lw.println("initImplementation();");
    d_lw.println("if (s_getEPV) (*s_getEPV)(epv);");
    d_lw.println("else {");
    d_lw.tab();
    d_lw.println("fputs(\"\\");
    d_lw.printlnUnformatted("Babel: Fatal error loading stage 2 Python implementation for " +
                            id.getFullName() + "\",");
    d_lw.println("stderr);");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.closeCxxExtern();
    d_lw.println();
  }

  private void writeGetSEPV()
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.openCxxExtern();
    d_lw.println("void");
    d_lw.println(IOR.getSetSEPVName(id) + "(" +
                 IOR.getSEPVName(id) + " *sepv) {");
    d_lw.tab();
    d_lw.println("initImplementation();");
    d_lw.println("if (s_getSEPV) (*s_getSEPV)(sepv);");
    d_lw.println("else {");
    d_lw.tab();
    d_lw.println("fputs(\"\\");
    d_lw.printlnUnformatted("Babel: Fatal error loading stage 2 Python implementation for " +
                            id.getFullName() + "\",");
    d_lw.println("stderr);");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.closeCxxExtern();
    d_lw.println();
  }

  private void writeCallLoad()
    throws CodeGenerationException
  {
    final SymbolID id = d_cls.getSymbolID();
    d_lw.println("void " + IOR.getSymbolName(id) + "__call_load(void) { ");
    d_lw.tab();
    d_lw.writeCommentLine("Python doesn't really need load");
    d_lw.backTab();
    d_lw.println("}");
    d_lw.println();
  }

  /**
   * Write functions to call each RMI fconnect and FGetURL function
   *
   * @param cls    the class for which an routine will be written.
   * @param writer the output writer to which the routine will be written.
   */
  private void writeRMIAccessFunctions() 
    throws CodeGenerationException
  {
    //Class cls = (Class) d_ext;
    SymbolID id = d_cls.getSymbolID();
    //TODO: When we RMI interfaces work, we should not exclude them! (edit getObjectDepe...)
    ArrayList dependencies = Utilities.sort(d_cls.getObjectDependencies());
    d_lw.openCxxExtern();
    for (Iterator i = dependencies.iterator(); i.hasNext(); ) {
      SymbolID destination_id = (SymbolID) i.next();

      //Forward declaration for the used functions
      d_lw.println();
  
      d_lw.println(C.getSymbolObjectPtr(destination_id) + " " + IOR.getSkelFConnectName(id,destination_id)+ 
                   "(const char* url, sidl_bool ar, "
                   + IOR.getExceptionFundamentalType() +
                   "*_ex) { ");
      d_lw.tab();

      //Actual call through to the pskels
      d_lw.println("return (*s_rmiExternals->f_" + Python.getPSkelFConnectName(id,destination_id)+")(url, ar, _ex);" );
      //d_lw.println("return "+C.getFullMethodName(destination_id, "_connectI")+"(url, ar, _ex);");
      //d_lw.println("return (*f_"+C.getFullMethodName(destination_id, "_connectI")+")(url, ar, _ex);");

      d_lw.backTab();
      d_lw.println("}");
      d_lw.println();

      d_lw.println(C.getSymbolObjectPtr(destination_id) + " " + 
                       IOR.getSkelFCastName(id,destination_id)+ 
                       "(void* bi, "+ IOR.getExceptionFundamentalType() + "*_ex) { ");
      d_lw.tab();
      d_lw.println("return (*s_rmiExternals->f_"+Python.getPSkelFCastName(id,destination_id)+")(bi, _ex);");
      d_lw.backTab();
      d_lw.println("}");
      d_lw.println();
    }
    d_lw.closeCxxExtern();
  }


  public synchronized void generateCode()
    throws CodeGenerationException
  {
    try {
      d_lw = Python.createLaunch(d_cls, "launch the implementation");
      writeIncludes();
      writeForwardDecls();
      writeStaticVars();
      writeInit();
      writeGetEPV();
      if (d_hasStatic) {
        writeGetSEPV();
      }
      writeCallLoad();
      writeRMIAccessFunctions(); 
    }
    finally {
      if (d_lw != null) {
        d_lw.close();
        d_lw = null;
      }
    }
  }

  private Collection argSymbols() throws CodeGenerationException {
    final String exceptionFundamentalName = BabelConfiguration.
      getBaseExceptionType();
    HashSet references = new HashSet();
    for(Iterator i = d_cls.getMethods(false).iterator(); i.hasNext() ; ){
      Method m = (Method)i.next();
      references.addAll(m.getSymbolReferences());
      if (!m.getThrows().isEmpty()) {
        references.add(Utilities.lookupSymbol(
         exceptionFundamentalName).getSymbolID());
      }
    }
    references.remove(d_cls.getSymbolID());
    return references;
  }

}
