//
// File:        RMI.java
// Package:     gov.llnl.babel.backend.rmi
// Copyright:   (c) 2004 The Regents of the University California
// Revision:     @(#) $Id: RMI.java 5151 2005-12-19 02:54:58Z leek2 $
// Description: 
// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// UCRL-CODE-2002-054
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

package gov.llnl.babel.backend.rmi;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.backend.CodeGenerationException;
import gov.llnl.babel.backend.IOR;
import gov.llnl.babel.backend.Utilities;
import gov.llnl.babel.backend.c.C;
import gov.llnl.babel.backend.writers.LanguageWriterForC;
import gov.llnl.babel.symbols.Argument;
import gov.llnl.babel.symbols.Extendable;
import gov.llnl.babel.symbols.SymbolTable;
import gov.llnl.babel.symbols.Type;

public class RMI { 

  /** generate the method call to pack an argument
   * @param lw              The language writer to write to
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param arg             The Argument itself
   * @param reuseable       True if (when the argument is an array) 
   *                         babel should attempt to copy the new data into
   *                         the old array.
   *
   */
  public static void packArg( LanguageWriterForC lw , String packer_typename, 
                              String packer_varname, Argument arg, boolean reuseable)
  { 
    packType( lw, packer_typename, packer_varname, arg.getType(), 
              arg.getFormalName(), arg.getFormalName(), arg.getMode(), 
              arg.isCopy(), reuseable, false);
    
  }

  /** generate the method call to pack an argument
   * @param lw              The language writer to write to
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param isCopy          Serialize the argument if it is an object.
   * @param reuseable       True is (when the argument is an array) 
   *                         babel should attempt to copy the new data into
   *                         the old array.
   * @param isReturn        If it's a return value (not an argument) 
   */
  public static void packType( LanguageWriterForC lw ,String packer_typename, 
                               String packer_varname,  Type type, 
                               String varname, String var, int mode, boolean isCopy, 
                               boolean reuseable, boolean isReturn) {
    
    switch(type.getDetailedType()) {
    case Type.CLASS:
    case Type.INTERFACE:
      packSymbol(lw, packer_typename, packer_varname, type, varname, var, mode,
                 isCopy, isReturn);
      break;
      
    case Type.ARRAY:
      packArray(lw, packer_typename, packer_varname, type, varname, var,
                mode, reuseable, isReturn);
      break;
    case Type.ENUM:
      type = new Type(Type.INT);
      //fall though
    default:
      String ext = RMI.getMethodExtension( type );
      String var_prefix = "";
      if(mode != Argument.IN) {
        var_prefix ="*";
      }
      lw.println(packer_typename + "_pack" + ext + "( " + packer_varname + ", \"" 
                 + varname + "\", " + var_prefix + var +", _ex);SIDL_CHECK(*_ex);");
    }
    return;
  }

  /** pack class or interface arguments
   * @param lw              The language writer to write to
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param type            The Type 
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param isCopy          Serialize the argument if it is an object.
   */
  private static void packSymbol(LanguageWriterForC lw, String packer_typename, 
                                 String packer_varname,  Type type, 
                                 String varname, String var, int mode, 
                                 boolean isCopy, boolean isReturn) {
    String var_prefix = "";
    String url_string = null; 
    if(mode != Argument.IN) {
      var_prefix ="*";
    }
    url_string = new String("sidl_BaseInterface__getURL" + "((sidl_BaseInterface)" 
                            + var_prefix + var + ", _ex)");

    if(isCopy) {
      if(RMI.isSerializable(type)) {
        packType(lw, packer_typename, packer_varname, new Type(Type.SYMBOL), 
                 varname, "sidl_io_Serializable__rmicast("+var_prefix+var+",_ex)", Argument.IN, isCopy, 
                 false, isReturn);
      }
    } else {
      lw.println("if("+var_prefix+var+"){");
      lw.tab();
      lw.println("char* _url = "+url_string+";SIDL_CHECK(*_ex);");
      lw.println(packer_typename + "_packString( " + packer_varname + ", \"" 
                 + varname + "\", _url, _ex);SIDL_CHECK(*_ex);");
      lw.println("free((void*)_url);");
      lw.backTab();
      lw.println("} else {");
      lw.tab();
      lw.println(packer_typename + "_packString( " + packer_varname + ", \"" 
                 + varname + "\", NULL, _ex);SIDL_CHECK(*_ex);");
      lw.backTab();
      lw.println("}");
    }
    return;
  }
  
  /** pack an array argument
   * @param lw              The language writer to write to
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param type            The Type 
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param reuseable       True is (when the argument is an array) 
   *                         babel should attempt to copy the new data into
   *                         the old array.
   * @param isReturn       If it's a return value (not an argument) 
   */
  private static void packArray(LanguageWriterForC lw, String packer_typename, 
                                String packer_varname,  Type type, 
                                String varname, String var, int mode, 
                                boolean reuseable, boolean isReturn) {
    String var_prefix = "";
    String ext = RMI.getMethodExtension( type );
    if(mode != Argument.IN && !isReturn) {
      var_prefix ="*";
    }
    if(isReturn || mode != Argument.INOUT) {
      reuseable = false;
    }
    if(RMI.isSerializable(type)) {
      lw.println(packer_typename + "_pack" + ext + "( " + packer_varname + ", \"" 
                 + varname + "\", " +var_prefix+ var +  
                 RMI.getEnsureIn(type,varname, reuseable)+", _ex);SIDL_CHECK(*_ex);");
    }
    
  }



  /** generate the method call to unpack an argument
   * @param lw              The language writer to write to
   * @param ext             The Extendable this is being generated in
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param inIOR           True if this is being generated in thie IOR
   */
  public static void unpackArg( LanguageWriterForC lw, Extendable ext, String packer_typename, 
                                String packer_varname, Argument arg, boolean inIOR ) 
    throws CodeGenerationException
  { 
    unpackType( lw, ext, packer_typename, packer_varname, arg.getType(), 
                arg.getFormalName(), arg.getFormalName(), arg.getMode(), 
                arg.isCopy(), false, inIOR );
  }

  /** generate the method call to unpack an argument or return value
   * @param lw              The language writer to write to
   * @param ext             The Extendable this is being generated in
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param type            The Type 
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param isCopy          Serialize the argument if it is an object.
   * @param isReturn        If it's a return value (not an argument) 
   * @param inIOR           True if this is being generated in thie IOR
   */
  public static void unpackType(LanguageWriterForC lw,  Extendable ext, String packer_typename, 
                                String packer_varname, Type type, 
                                String varname, String var, int mode, 
                                boolean isCopy, boolean isReturn, boolean inIOR)
    throws CodeGenerationException {
 
    switch(type.getDetailedType()) {
    case Type.CLASS:
    case Type.INTERFACE:
      unpackSymbol(lw, ext, packer_typename, packer_varname, type, varname, var, mode, 
                   isCopy, isReturn, inIOR);
      break;
    case Type.ARRAY:
      unpackArray(lw, packer_typename, packer_varname, type, varname, var, mode, isReturn);
      break;        
    case Type.ENUM:
      unpackEnum(lw, packer_typename, packer_varname, type, varname, var, mode, 
                 isReturn, inIOR); 
      break;
    default:
      String m_ext = RMI.getMethodExtension( type );
      String var_prefix = "";
      if(mode == Argument.IN || isReturn) {
        var_prefix = "&";
      }
      lw.println(packer_typename + "_unpack" + m_ext + "( " + packer_varname 
                 + ", \"" + varname + "\", " + var_prefix + var +", _ex);SIDL_CHECK(*_ex);");
      break;
    }
    return; 
      
  }

  /** unpack class or interface arguments
   * @param lw              The language writer to write to
   * @param ext             The Extendable this is being generated in
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param type            The Type 
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param isCopy          Serialize the argument if it is an object.
   * @param isReturn        If it's a return value (not an argument) 
   * @param inIOR           True if this is being generated in thie IOR
   */
  private static void unpackSymbol(LanguageWriterForC lw, Extendable ext, String packer_typename, 
                                   String packer_varname,  Type type, 
                                   String varname, String var, int mode, 
                                   boolean isCopy, boolean isReturn, boolean inIOR) 
    throws CodeGenerationException 
  {

    String raddref = "FALSE";
    String mod = "";
    if(mode != Argument.IN) {
      mod = "*";
    }
    if(mode == Argument.IN && !isReturn) {  //We should only raddref on in args
      raddref = "TRUE";
    }

    if(isCopy) {
      if(RMI.isSerializable(type)) {
        unpackType(lw, ext, packer_typename, packer_varname, new Type(Type.SYMBOL), 
                   varname, var+"_ser", Argument.IN, isCopy, isReturn, inIOR);
        if(inIOR) {
          lw.println(mod+var +" = "+IOR.getSkelFCastName(ext.getSymbolID(), type.getSymbolID())+ 
                     "((sidl_BaseInterface)"+var+"_ser, _ex);SIDL_CHECK(*_ex);");
        } else {
          lw.println(mod+var +" = "+IOR.getSymbolName(type.getSymbolID())+ "__rmicast" +
                     "((sidl_BaseInterface)"+var+"_ser, _ex);");
        }
      }
      return;
    }
    
    RMI.unpackType(lw, ext, packer_typename, packer_varname, new Type(Type.STRING), 
                   varname,var+"_str", Argument.IN, inIOR, isCopy, false);
    
    
    if(inIOR) {
      lw.println(mod+var +" = "+IOR.getSkelFConnectName(ext.getSymbolID(), type.getSymbolID())+ 
                 "("+var+"_str, "+raddref+", _ex);SIDL_CHECK(*_ex);");
    } else {
      lw.println(mod+var +" = "+ IOR.getRemoteConnectName(type.getSymbolID())+ 
                 "("+var+"_str, "+raddref+", _ex);SIDL_CHECK(*_ex);");
    }
    
  }

  private static void unpackEnum(LanguageWriterForC lw, String packer_typename, 
                                   String packer_varname,  Type type, 
                                   String varname, String var, int mode, 
                                   boolean isReturn, boolean inIOR) 
    throws CodeGenerationException{

    String mod = "";
    if(mode != Argument.IN) {
      mod = "*";
    }
    
    RMI.unpackType(lw, null, packer_typename, packer_varname, new Type(Type.INT), 
                   varname,var+"_tmp", Argument.IN, inIOR, false, false);
    lw.println(mod+var +" = ("+IOR.getReturnString(type,true,true)+")"+var+"_tmp;");
    
  }


  /** unpack array arguments
   * @param lw              The language writer to write to
   * @param packer_typename The packer type (often sidl_rmi_Serializer)
   * @param packer_varname  The name of the packer object
   * @param type            The Type 
   * @param varname         The name the argument is called in the
   *                        serializer
   * @param var             The actual argument name (return = _retval)
   * @param mode            IN, OUT, or INOUT
   * @param isReturn        If it's a return value (not an argument) 
   */
  private static void unpackArray(LanguageWriterForC lw, String packer_typename, 
                                  String packer_varname,  Type type, 
                                  String varname, String var, int mode, 
                                  boolean isReturn) {
    
    String ext = RMI.getMethodExtension( type );
    String var_prefix = "";
    if(mode == Argument.IN || isReturn) {
      var_prefix = "&";
    }
    
    if(RMI.isSerializable(type)) {
      lw.println(packer_typename + "_unpack" + ext + "( " + packer_varname 
                 + ", \"" + varname + "\", " + var_prefix + var + 
                 RMI.getEnsureOut(type)+ ", _ex);SIDL_CHECK(*_ex);");
    }
  }


  /** generate the proper method extension for packing or unpacking
   * @param t
   */
  public static String getMethodExtension( Type t ) { 
    if ( t.isPrimitive() ) { /* fixed size & string */
      String buff = t.getTypeString();  
      return buff.substring(0,1).toUpperCase() + buff.substring(1); 
    } else if ( t.isArray() ) { 
      Type arrayType = t.getArrayType();
      if(arrayType == null) {
        return "GenericArray";
      }
      return getMethodExtension(t.getArrayType()) + "Array";

    } else if(t.isSymbol()) {
      return "Serializable";
    } else { 
      return "ERROR";
    }
  }

  /**
   * Unpacking array arguments requires a couple of extra args, generate
   * those.
   * @param t
   */
  private static String getEnsureOut(Type t) {
    String ret = "";
    String israrray = null;
    if(t.getArrayType() != null) {
      if(t.isArray()) {
        if(t.isRarray()) {
          israrray = "TRUE";
        } else {
          israrray = "FALSE";
        }
        if(t.hasArrayOrderSpec()) {
          ret = ","+BabelConfiguration.getArrayOrderName(t.getArrayOrder())+","+t.getArrayDimension()+","+israrray;
        } else {
          ret = ",0,0,"+israrray;
        }
      }
    }
    return ret;
  }

  /**
   * Packing array arguments requires a couple of extra args, generate
   * those.
   * @param t
   */
  private static String getEnsureIn(Type t, String varname, boolean check_reuse) {
    String ret = "";
    if(t.getArrayType() != null) {
      if(t.hasArrayOrderSpec()) {
        ret = ","+BabelConfiguration.getArrayOrderName(t.getArrayOrder())+","+t.getArrayDimension();
      } else {
        ret = ",0,0";
      }
    }
    if(check_reuse) {
      String reuseFlag = "(*"+varname+"==" +varname+RMI.getDataExt()+")";
      ret = ret + "," + reuseFlag;
    } else {
      ret = ret + ",0";
    }

    return ret;
  }

  public static void declareStackArgs(LanguageWriterForC lw , Argument arg) throws CodeGenerationException {
    switch(arg.getType().getDetailedType()) {
    case Type.CLASS:
    case Type.INTERFACE:
      declareStackSymbol(lw, arg.getType(), arg.getFormalName(), arg.getMode(), arg.isCopy(), false);
      break;
    case Type.ENUM:
      if(arg.getMode() != Argument.OUT) {
        lw.println(IOR.getReturnString(new Type(Type.INT)) + " "
                   + arg.getFormalName() + "_tmp= 0;");
      }
      if(arg.getMode() == Argument.IN) {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + 
                   Utilities.getTypeInitialization(arg.getType()) + ";");
      } else {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + RMI.getDataExt() + Utilities.getTypeInitialization(arg.getType()) + ";");
        lw.println(IOR.getArgumentWithFormal(arg, true, false, false) 
                   + " = &"+ arg.getFormalName()+RMI.getDataExt()+";");
      }
      break;
    case Type.STRING:
      if(arg.getMode() == Argument.IN) {
        //Special case for string: normally IN strings are const, not in RMI
        lw.println(IOR.getReturnString(arg.getType()) + " " 
                   + arg.getFormalName() + "= "+C.NULL+";");
      } else {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + RMI.getDataExt()+ Utilities.getTypeInitialization(arg.getType()) +";");
        lw.println(IOR.getArgumentWithFormal(arg, true, false, false) 
                   + " = &"+ arg.getFormalName()+RMI.getDataExt()+";");
      }
      break;
    case Type.ARRAY:
      if(arg.getMode() == Argument.IN) {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + " = " + C.NULL + ";");
      } else {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + RMI.getDataExt()+ " = " + C.NULL + ";");
        lw.println(IOR.getArgumentWithFormal(arg, true, false, false) 
                   + " = &"+ arg.getFormalName()+RMI.getDataExt()+";");
      }
      break;
    default:
      if(arg.getMode() == Argument.IN) {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) +
                   Utilities.getTypeInitialization(arg.getType()) + ";");
      } else {
        lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + RMI.getDataExt()+ Utilities.getTypeInitialization(arg.getType()) +";");
        lw.println(IOR.getArgumentWithFormal(arg, true, false, false) 
                   + " = &"+ arg.getFormalName()+RMI.getDataExt()+";");
      }
      
    }
    /*
      } else {
      if(arg.getMode() == Argument.IN) {
      if(arg.getType().getDetailedType() == Type.STRING) {
      //Special case for string: normally IN strings are const, not in RMI
      lw.println(IOR.getReturnString(arg.getType()) + " " 
      + arg.getFormalName() + "= "+C.NULL+";");
      } else {
      lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + ";");
      }
      } else {
      lw.println(IOR.getArgumentWithFormal(arg, true, false, true) + RMI.getDataExt()+";");
      lw.println(IOR.getArgumentWithFormal(arg, true, false, false) 
      + " = &"+ arg.getFormalName()+RMI.getDataExt()+";");
      }
      }
    */
  }
    
  

  public static void declareStackReturn(LanguageWriterForC lw , Type type, boolean isCopy) 
    throws CodeGenerationException {
    if(type.getDetailedType() == Type.CLASS ||
       type.getDetailedType() == Type.INTERFACE) {
      declareStackSymbol(lw, type, RMI.getReturnArgName(), Argument.IN, isCopy, true);
    } else {
      lw.println( IOR.getReturnString(type, true, false) + " " + RMI.getReturnArgName()+Utilities.getTypeInitialization(type)+";");
    }
  }
  
  public static void declareStackSymbol(LanguageWriterForC lw , Type type, 
                                        String var, int mode, boolean isCopy, boolean isReturn) 
    throws CodeGenerationException {
    
    if(mode != Argument.OUT) {
      if(isCopy) {
        lw.println("sidl_io_Serializable " +var+ RMI.getSerializableExt()+" = "+C.NULL+ ";");
      } else {
        if(!isReturn) {
          //We need a string to unserialize the url into
          lw.println(IOR.getReturnString(new Type(Type.STRING)) 
                     +" "+var + RMI.getStringExt() +" = "+C.NULL+";");
        }
      }
    }
    if(mode == Argument.IN) {
      lw.println(C.getSymbolObjectPtr(type.getSymbolID()) + " " 
                 + var + " = "+C.NULL+";");
    } else {
      lw.println(IOR.getReturnString(type) + " " 
                 + var + RMI.getDataExt() +" = "+C.NULL+";");
      lw.println(IOR.getReturnString(type) + "* " 
                 + var + " = &"+var+RMI.getDataExt()+";");
    }
  }



  /**
   * Returns the varialbe extension used to identify a temporary variable of
   * type sidl.io.Serializable.  (Used in object serialization)
   */
  public static String getSerializableExt() {
    return "_ser";
  }
  /**
   * Returns the varialbe extension used to identify a temporary variable of
   * type String to be used for object connection.
   */
  public static String getStringExt() {
    return "_str";
  }
  /**
   * Returns the varialbe extension used to identify a temporary variable of
   * for holding the pointer that inout arguments point to in serialization.
   *
   * ie: if we have an inout argument of type int, it is passed as an
   * int32_t*.  However, we must unserialize the actual integer somewhere. So:
   * int32_t x_data = unserialize_int();
   * int32_t* x = &x_data;
   * _retval = foo(x); 
   */
  public static String getDataExt() {
    return "_data";
  }

  /**
   * Returns internal name of the return value.
   */
  public static String getReturnArgName() {
    return "_retval";
  }

  /**
   * Classes that do no inherit from serializable and arrays of the same
   * are not serializable.  (There's gotta be a better way to do this...)
   */
  private static boolean isSerializable(Type type) {
    SymbolTable table = SymbolTable.getInstance();
    Extendable serializable = (Extendable)table.lookupSymbol("sidl.io.Serializable");
    
    if(type.getDetailedType() == Type.CLASS ||
       type.getDetailedType() == Type.INTERFACE) {
      Extendable this_ext = (Extendable)table.lookupSymbol(type.getSymbolID().getFullName()); 
      if(!this_ext.hasAncestor(serializable)) {
        return false;
      }
    }

    if((type.getArrayType() != null) &&
       (type.getArrayType().getDetailedType() == Type.CLASS ||
        type.getArrayType().getDetailedType() == Type.INTERFACE)) {
      Extendable this_ext = (Extendable)table.lookupSymbol(type.getArrayType().getSymbolID().getFullName()); 
      //if(!this_ext.hasAncestor(serializable)) {
      if(this_ext.getSymbolID().getFullName().compareTo(serializable.getSymbolID().getFullName()) != 0) {
        return false;
      }
    }
    return true;
  }

  public static String LangSpecificInit() {
    return "LANG_SPECIFIC_INIT()";
  }
}
