!
! File:       argstest.F90
! Copyright:  (c) 2002 The Regents of the University of California
! Revision:   @(#) $Revision: 5121 $
! Date:       $Date: 2005-12-09 14:41:13 -0800 (Fri, 09 Dec 2005) $
! Description:Exercise the FORTRAN interface
!
!
#include "Args_Cfloat_fAbbrev.h"
#include "Args_Cdouble_fAbbrev.h"
#include "Args_Cint_fAbbrev.h"
#include "Args_Clong_fAbbrev.h"
#include "Args_Cdcomplex_fAbbrev.h"
#include "Args_Cfcomplex_fAbbrev.h"
#include "Args_Cbool_fAbbrev.h"
#include "Args_Cchar_fAbbrev.h"
#include "synch_RegOut_fAbbrev.h"
#include "synch_ResultType_fAbbrev.h"

subroutine starttest(number)
  use sidl
  use synch_RegOut
  use sidl_BaseInterface
  implicit none
  integer (kind=sidl_int) :: number
  type(synch_RegOut_t) :: tracker
  type(sidl_BaseInterface_t) :: throwaway_exception
  call getInstance(tracker, throwaway_exception)
  call startPart(tracker, number, throwaway_exception)
  call deleteRef(tracker, throwaway_exception)
end subroutine starttest

subroutine reporttest(test, number, python)
  use sidl
  use synch_RegOut
  use sidl_BaseInterface
  use synch_ResultType
  implicit none
  integer (kind=sidl_int) :: number
  logical                             :: test, python
  type(synch_RegOut_t) :: tracker
  type(sidl_BaseInterface_t) :: throwaway_exception
  call getInstance(tracker, throwaway_exception)
  if (test) then
     call endPart(tracker, number, PASS, throwaway_exception)
  else
     if (python) then
        call endPart(tracker, number, XFAIL, throwaway_exception)
     else
        call endPart(tracker, number, FAIL, throwaway_exception)
     endif
  endif
  call deleteRef(tracker, throwaway_exception)
  number = number + 1
end subroutine reporttest

subroutine testbool(test)
  use sidl
  use Args_Cbool
  use sidl_BaseInterface
  implicit none
  type(Args_Cbool_t) :: obj
  integer (kind=sidl_int)  :: test
  logical                         :: out, inout, retval
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = .true.
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, retval, throwaway_exception)
  call reporttest(retval, test,  .false.)
  call starttest(test)
  call passin(obj, .true., retval, throwaway_exception)
  call reporttest(retval, test,  .false.)
  call starttest(test)
  call passout(obj, out, retval, throwaway_exception)
  call reporttest(retval .and. out, test,  .false.)
  call starttest(test)
  call passinout(obj, inout, retval, throwaway_exception)
  call reporttest(retval .and. .not. inout, test,  .false.)
  call starttest(test)
  call passeverywhere(obj, .true., out, inout, retval, throwaway_exception)
  call reporttest(retval .and. out .and. inout, test,  &
                  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testbool

subroutine testint(test)
  use sidl
  use Args_Cint
  use sidl_BaseInterface
  implicit none
  type(Args_Cint_t) :: obj
  integer (kind=sidl_int)  :: test 
  logical                         :: bretval
  integer (kind=sidl_int)  :: iretval, out, inout
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = 3
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, iretval, throwaway_exception)
  call reporttest(iretval .eq. 3, test,  .false.)
  call starttest(test)
  call passin(obj, 3, bretval, throwaway_exception)
  call reporttest(bretval, test,  .false.)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. 3), test,  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. -3), test,  &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, 3, out, inout, iretval, throwaway_exception)
  call reporttest((iretval .eq. 3) .and.  (out .eq. 3) .and. &
                  (inout .eq. 3), test,  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testint

subroutine testchar(test)
  use sidl
  use Args_Cchar
  use sidl_BaseInterface
  implicit none
  type (Args_Cchar_t) :: obj
  integer (kind=sidl_int)  :: test
  logical                         :: bretval
  character (len=1)               :: cretval, out, inout
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = 'A'
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, cretval, throwaway_exception)
  call reporttest(cretval .eq. '3', test,  .false.)
  call starttest(test)
  call passin(obj, '3', bretval, throwaway_exception)
  call reporttest(bretval, test,  .false.)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. '3'), test,  &
                  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. 'a'), test,  &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, '3', out, inout, cretval, throwaway_exception)
  call reporttest((cretval .eq. '3') .and.  (out .eq. '3') .and. &
                  (inout .eq. 'A'), test,  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testchar

subroutine testlong(test)
  use sidl
  use Args_Clong
  use sidl_BaseInterface
  implicit none
  type(Args_Clong_t) :: obj
  integer (kind=sidl_int)  :: test
  logical                         :: bretval
  integer (kind=sidl_long) :: out, inout, iretval, inval
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = 3
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, iretval, throwaway_exception)
  call reporttest(iretval .eq. 3, test,  .false.)
  call starttest(test)
  inval = 3
  call passin(obj, inval, bretval, throwaway_exception)
  call reporttest(bretval, test,  .false.)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. 3), test,  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. -3), test,  &
                  .false.)
  call starttest(test)
  inval = 3
  call passeverywhere(obj, inval, out, inout, iretval, throwaway_exception)
  call reporttest((iretval .eq. 3) .and.  (out .eq. 3) .and. &
                  (inout .eq. 3), test,  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testlong

subroutine testfloat(test,  python)
  use sidl
  use Args_Cfloat
  use sidl_BaseInterface
  implicit none
  type(Args_Cfloat_t) :: obj
  integer (kind=sidl_int)  :: test
  logical                         :: bretval, python
  real (kind=sidl_float) :: out, inout, fretval
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = 3.1
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, fretval, throwaway_exception)
  call reporttest(fretval .eq. 3.1, test,  .false.)
  call starttest(test)
  call passin(obj, 3.1, bretval, throwaway_exception)
  call reporttest(bretval, test,  python)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. 3.1), test,  &
                  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. -3.1), test,  &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, 3.1, out, inout, fretval, throwaway_exception)
  call reporttest((fretval .eq. 3.1) .and.  (out .eq. 3.1) .and. &
                  (inout .eq. 3.1), test,  python)
  call deleteRef(obj, throwaway_exception)
end subroutine testfloat

subroutine testdouble(test)
  use sidl
  use Args_Cdouble
  use sidl_BaseInterface
  implicit none
  type(Args_Cdouble_t) :: obj
  integer (kind=sidl_int)    :: test
  logical                           :: bretval
  real (kind=sidl_double) :: out, inout, dretval
  type(sidl_BaseInterface_t) :: throwaway_exception

  inout = 3.14d0
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, dretval, throwaway_exception)
  call reporttest(dretval .eq. 3.14d0, test,  .false.)
  call starttest(test)
  call passin(obj, 3.14d0, bretval, throwaway_exception)
  call reporttest(bretval, test,  .false.)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. 3.14d0), test,  &
                  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. -3.14d0), test,  &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, 3.14d0, out, inout, dretval, throwaway_exception)
  call reporttest((dretval .eq. 3.14d0) .and.  (out .eq. 3.14d0) .and. &
                  (inout .eq. 3.14d0), test,  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testdouble

subroutine testfcomplex(test,  python)
  use sidl
  use Args_Cfcomplex
  use sidl_BaseInterface
  implicit none
  type(Args_Cfcomplex_t) :: obj
  integer (kind=sidl_int)     :: test
  logical                            :: bretval, python
  complex (kind=sidl_fcomplex) :: in, out, inout, cretval
  type(sidl_BaseInterface_t) :: throwaway_exception

  in = (3.1,3.1)
  inout = (3.1, 3.1)
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, cretval, throwaway_exception)
  call reporttest(cretval .eq. (3.1,3.1), test,  .false.)
  call starttest(test)
  call passin(obj, in, bretval, throwaway_exception)
  call reporttest(bretval, test,  python)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. (3.1,3.1)), test,  &
                  .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. (3.1,-3.1)), test, &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, in, out, inout, cretval, throwaway_exception)
  call reporttest((cretval .eq. (3.1,3.1)) .and.  (out .eq. (3.1,3.1)) .and. &
                  (inout .eq. (3.1,3.1)), test,  python)
  call deleteRef(obj, throwaway_exception)
end subroutine testfcomplex

subroutine testdcomplex(test)
  use sidl
  use Args_Cdcomplex
  use sidl_BaseInterface
  implicit none
  type(Args_Cdcomplex_t) :: obj
  integer (kind=sidl_int)       :: test
  logical                              :: bretval
  complex (kind=sidl_dcomplex) :: in, out, inout, cretval
  type(sidl_BaseInterface_t) :: throwaway_exception

  in = (3.14d0,3.14d0)
  inout = (3.14d0, 3.14d0)
  call new(obj, throwaway_exception)
  call starttest(test)
  call returnback(obj, cretval, throwaway_exception)
  call reporttest(cretval .eq. (3.14d0,3.14d0), test,  &
                  .false.)
  call starttest(test)
  call passin(obj, in, bretval, throwaway_exception)
  call reporttest(bretval, test,  .false.)
  call starttest(test)
  call passout(obj, out, bretval, throwaway_exception)
  call reporttest(bretval .and. (out .eq. (3.14d0,3.14d0)), test, &
                   .false.)
  call starttest(test)
  call passinout(obj, inout, bretval, throwaway_exception)
  call reporttest(bretval .and. (inout .eq. (3.14d0,-3.14d0)), test, &
                  .false.)
  call starttest(test)
  call passeverywhere(obj, in, out, inout, cretval, throwaway_exception)
  call reporttest((cretval .eq. (3.14d0,3.14d0)) .and.  &
                  (out .eq. (3.14d0,3.14d0)) .and. &
                  (inout .eq. (3.14d0,3.14d0)), test,   &
                  .false.)
  call deleteRef(obj, throwaway_exception)
end subroutine testdcomplex


program argstest
  use sidl
  use synch_RegOut
  use sidl_BaseInterface
  integer (kind=sidl_int) :: test
  character (len=80)             :: language
  type(synch_RegOut_t) :: tracker
  type(sidl_BaseInterface_t) :: throwaway_exception
  language = ' '
  if (IArgc() .eq. 1) then
     call GetArg(1, language)
  endif
  call getInstance(tracker, throwaway_exception)
  test = 1
  call setExpectations(tracker, 40, throwaway_exception)
  call writeComment(tracker, 'Boolean tests', throwaway_exception)
  call testbool(test)
  call writeComment(tracker, 'Character tests', throwaway_exception)
  call testchar(test)
  call writeComment(tracker, 'Integer tests', throwaway_exception)
  call testint(test)
  call writeComment(tracker, 'Long tests', throwaway_exception)
  call testlong(test)
  call writeComment(tracker, 'Float tests', throwaway_exception)
  call testfloat(test,  language .eq. 'Python')
  call writeComment(tracker, 'Double tests', throwaway_exception)
  call testdouble(test)
  call writeComment(tracker, 'Fcomplex tests', throwaway_exception)
  call testfcomplex(test,  language .eq. 'Python')
  call writeComment(tracker, 'Dcomplex tests', throwaway_exception)
  call testDcomplex(test)
  call close(tracker, throwaway_exception)
  call deleteRef(tracker, throwaway_exception)
end program argstest
